--
-- PickupChain
-- Specialization for the pickup chain
--
-- @author  Stefan Maurus	
-- @date  13/02/15

-- Copyright © Stefan Maurus, www.stefanmaurus.de

--[[
	<pickupChainAnimation index1="INDEX" index2="INDEX" maxDistance="FLOAT">
		<part index="INDEX">
			<key time="0" rotX="FLOAT"/>
			..
			<key time="1" rotX="FLOAT"/>				
		</part>
		<part index="INDEX">
			<key time="0" rotX="FLOAT"/>
			..
			<key time="1" rotX="FLOAT"/>				
		</part>
		..
	</pickupChainAnimation>
]]


PickupChain = {};

function PickupChain.prerequisitesPresent(specializations)
	return true;
end;

function PickupChain:load(xmlFile)
	self.pickupChain = {};
	self.pickupChain.index1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pickupChainAnimation#index1"));
	self.pickupChain.index2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pickupChainAnimation#index2"));
	self.pickupChain.maxDistance = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pickupChainAnimation#maxDistance"), 0.7);
	
	local i = 0;
	self.pickupChain.animCurves = {};
	while true do 
		local key = string.format("vehicle.pickupChainAnimation.part(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
		
		local animCurve = AnimCurve:new(linearInterpolator1);

		local j = 0;
		while true do 
			local key2 = string.format(key..".key(%d)", j);
			if not hasXMLProperty(xmlFile, key2) then
				break;
			end;
			
			local t = getXMLFloat(xmlFile, key2.."#time");
			local xRot = getXMLFloat(xmlFile, key2.."#rotX");	
			
			animCurve:addKeyframe({v=xRot, time=t});
			j = j + 1;	
		end;
		
		table.insert(self.pickupChain.animCurves, {index=index, animCurve=animCurve});
		i = i + 1;		
	end;
end;

function PickupChain:delete()
end;

function PickupChain:readStream(streamId, connection)
end;

function PickupChain:writeStream(streamId, connection) 
end;

function PickupChain:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PickupChain:keyEvent(unicode, sym, modifier, isDown)
end;

function PickupChain:update(dt)
end;

function PickupChain:updateTick(dt)
	local x, y, z = getWorldTranslation(self.pickupChain.index1);
	local x1, y1, z1 = getWorldTranslation(self.pickupChain.index2);
	local distance = Utils.vector3Length(x-x1, y-y1, z-z1)+0.1;
	local percent = math.max(math.min(distance / self.pickupChain.maxDistance, 1), 0);
	
	for _, part in pairs(self.pickupChain.animCurves) do
		local x = part.animCurve:get(percent);
		setRotation(part.index, math.rad(x), 0, 0);
	end;	
	
	local x, y, z = getRotation(Utils.indexToObject(self.components, "0>1|3|0|0|0|0|0"));
	--print(x.." "..y.." "..z);
end;

function PickupChain:draw()
end;

function PickupChain:onAttach()
end;

function PickupChain:onDetach()	
end; 